static const char *RcsId = "$Header: /home/cvsadm/cvsroot/fermi/servers/plcesa/src/PLCesall.cpp,v 1.15 2018-07-24 13:03:34 graziano Exp $";
//+=============================================================================
//
// file :         PLCesall.cpp
//
// description :  C++ source for the PLCesall and its commands. 
//                The class is derived from Device. It represents the
//                CORBA servant object which will be accessed from the
//                network. All commands which can be executed on the
//                PLCesall are implemented in this file.
//
// project :      TANGO Device Server
//
// $Author: graziano $
//
// $Revision: 1.15 $
//
// $Log: PLCesall.cpp,v $
// Revision 1.15  2018-07-24 13:03:34  graziano
// new padres valves
//
// Revision 1.14  2017-10-06 08:11:31  graziano
// changed magnedyn command code from 808 to 909
//
// Revision 1.13  2017-08-21 08:28:37  graziano
// tango 9.2.5a
//
// Revision 1.12  2016-04-05 08:48:38  graziano
// fixed dst
//
// Revision 1.11  2015-06-03 15:16:24  graziano
// added timer
//
// Revision 1.10  2013-01-07 10:42:35  graziano
// added command to set vacuum interlock threshold
//
// Revision 1.9  2012-10-01 09:16:08  graziano
// added commands for PPM1 and PPM2
//
// Revision 1.8  2012-08-09 09:22:40  graziano
// fixed
//
// Revision 1.7  2012-02-09 10:57:17  graziano
// updated for gas padres
//
// Revision 1.6  2012-01-16 10:34:48  graziano
// modified gas padres
//
// Revision 1.5  2011/07/04 08:13:41  graziano
// fixed for new valves
//
// Revision 1.4  2011/04/22 08:25:36  graziano
// updated mysql log
//
// Revision 1.3  2011/03/25 11:04:48  graziano
// fixed acknowledge
//
// Revision 1.2  2010/12/03 14:59:02  graziano
// new log message queue,
// improved startup time,
// added shutter
//
// Revision 1.1  2010/11/23 11:01:23  graziano
// development
//
//
//
//
// copyleft :     European Synchrotron Radiation Facility
//                BP 220, Grenoble 38043
//                FRANCE
//
//-=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================



//===================================================================
//
//	The following table gives the correspondence
//	between commands and method name.
//
//  Command name |  Method name
//	----------------------------------------
//  State        |  dev_state()
//  Status       |  dev_status()
//  Send_cmd     |  send_cmd()
//  Acknowledge  |  acknowledge()
//
//===================================================================


#include <tango.h>
#include <PLCesall.h>
#include <PLCesallClass.h>
#include "plclgthread.h"
#include "plcsrvthread.h"
#include <iomanip>      //used for debug dump of db

namespace PLCesall_ns
{

//+----------------------------------------------------------------------------
//
// method : 		PLCesall::PLCesall(string &s)
// 
// description : 	constructor for simulated PLCesall
//
// in : - cl : Pointer to the DeviceClass object
//      - s : Device name 
//
//-----------------------------------------------------------------------------
PLCesall::PLCesall(Tango::DeviceClass *cl,string &s)
:TANGO_BASE_CLASS(cl,s.c_str())
{
	init_device();
}

PLCesall::PLCesall(Tango::DeviceClass *cl,const char *s)
:TANGO_BASE_CLASS(cl,s)
{
	init_device();
}

PLCesall::PLCesall(Tango::DeviceClass *cl,const char *s,const char *d)
:TANGO_BASE_CLASS(cl,s,d)
{
	init_device();
}
//+----------------------------------------------------------------------------
//
// method : 		PLCesall::delete_device()
// 
// description : 	will be called at device destruction or at init command.
//
//-----------------------------------------------------------------------------
void PLCesall::delete_device()
{
	//	Delete device's allocated object
	abortflag = 1; //stops thread
	timest_vect to_log;
	to_log.logtype = L_EXIT;
	logloop->raw_list.push_back(to_log);
	sleep(1);

	for(int k=0;k<MAX_N_ATTR;k++)
	{
			delete [] val_labels[k];
			delete [] val_pos[k];
	}
	
	delete [] attr_Real_system_read;

	if(sock != -1)
	{
		close(sock);
		sock = -1;
	}
	if(new_sd)
		close(new_sd);
	//delete mutex;
  	mysql_close(&dbp);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCesall::init_device()
// 
// description : 	will be called at device initialization.
//
//-----------------------------------------------------------------------------
void PLCesall::init_device()
{
	INFO_STREAM << "PLCesall::PLCesall() create device " << device_name << endl;

	set_change_event("Stat_system", true, false);		//attribute name, implemented, detect, if not setting detect=false have to set abs_change or rel_change for each attribute
	set_change_event("Alarm_system", true, false);		//attribute name, implemented, detect, if not setting detect=false have to set abs_change or rel_change for each attribute
	set_change_event("Real_system", true, false);		//attribute name, implemented, detect, if not setting detect=false have to set abs_change or rel_change for each attribute
	set_change_event("Db_alarm", true, false);		//attribute name, implemented, detect, if not setting detect=false have to set abs_change or rel_change for each attribute

	// Initialise variables to default values
	//--------------------------------------------
	get_device_property();
	Socketds = NULL;
//	plc_thread = NULL;
	logstate = true;//logging is active
	
	attr_Real_system_read = new Tango::DevDouble[DB55_DATA_SIZE];

//init containers for server data

	for(int k=0;k<MAX_N_ATTR;k++)
	{
		att_names[k] = "";
		att_len[k] = 0;
	}
	att_names[0] = "Stat_system"; //50
	att_len[0] = MESG_STAT_SYS_LEN;
	att_names[1] = "Alarm_system"; //53
	att_len[1] = MESG_ALM_SYS_LEN;
	att_names[2] = "Db_alarm"; //53
	att_len[2] = MESG_ALM_SYS_LEN;
	att_names[3] = "Out_system"; //54
	att_len[3] = MESG_OUT_SYS_LEN;
	att_names[4] = "Real_system"; //55
	att_len[4] = MESG_REAL_SYS_LEN;

	for(int k=0;k<MESG_STAT_SYS_LEN;k++){
		msg_statsys[k] = true;
	}
	for(int k=0;k<MESG_ALM_SYS_LEN;k++){
		msg_almsys[k] = true;
	}
	for(int k=0;k<MESG_OUT_SYS_LEN;k++){
		msg_outsys[k] = true;
	}
	for(int k=0;k<MESG_REAL_SYS_LEN;k++){
		msg_realsys[k] = 0.0;
	}
	
	for(int k=0;k<MAX_N_ATTR;k++){
		val_labels[k] = new string[MAX_ALL];
		val_pos[k] = new string[MAX_ALL];
		for(int j=0;j<MAX_ALL;j++){
			(val_labels[k])[j] = "|||";
			(val_pos[k])[j] = "---";
		}
	}

	for(int k=0;k<MESG_ALM_SYS_LEN;k++){//at the beginning all the alarms are new, mask is empty
		alarm_db[k] = 0;//no active alarm
		w_alarm_db[k] = 0;
		alm_active[k] = false;
		old_alm[k] = true;
	}


        timestamp_stat = 0;
        timestamp_alm = 0;

        timestamp_last = 0;

//	read data filename
	//	start_file= "/elettra_log/logs/plcll-srv.labels.txt"; //name of the startup file
	vector<string>::iterator iter = data_config.begin();
	start_file = ((string)*iter).c_str();
	iter++;
/*	cout<<"loading srv config from file: "<< start_file <<endl;
	load_cond();
	cout<<"loaded info from file: "<< start_file <<endl;*/


	//init_attributeprop();


//read info for socket connection
	if(connect_param.size() ==2)
	{
		vector<string>::iterator iter2 = connect_param.begin();
		ipaddr = ((string)*iter2);
		iter2++;
		pport = atoi(((string)*iter2).c_str());
		iter2++;
	}
	else
	{
		ipaddr = remote_ip;
		pport = local_port;
	}
	//find hostname
	char indi[INET_ADDRSTRLEN];
	struct hostent* he;
	string nome = Tango::Util::instance()->get_host_name();
	
        he = gethostbyname(nome.c_str());
        if(he == NULL){
		WARN_STREAM << "PLCesall::Error getting host names" << endl;
        }
	int i = 0;
        while(he->h_addr_list[i] != NULL){
                if(inet_ntop(AF_INET, he->h_addr_list[i], indi, INET_ADDRSTRLEN) <= 0){
                        perror("inet_ntop");
                }
		INFO_STREAM << "PLCesall::found host name: "<<  indi << endl;
		i++;
	}
	srv_host = indi;
//init database connection
	DEBUG_STREAM << "PLCesall::init_device() init database connection" << endl;
	init_database();

	logloop = new plclgthread(this);
	logloop->start();


	timeval t_now;
	//	Add your own code to control device here
	try {
		load_cond();
		gettimeofday(&t_now,NULL);//takes timestamp
		timest_vect to_log;
		to_log.logtype = L_UPDATE;
		to_log.tstamp = (double)t_now.tv_sec+(((double)t_now.tv_usec)/1000000);
;
		logloop->raw_list.push_back(to_log);
		//init_attributeprop();
	}
	catch (Tango::DevFailed e)
	{
		INFO_STREAM << "PLCesall::init_device(): error updating description="<<e.errors[0].desc << endl;
	}

//log to db the server restart
	DEBUG_STREAM << "PLCesall::init_device() log to db srv restart" << endl;
//	timeval t_now;
//	gettimeofday(&t_now,NULL);//takes timestamp

	timest_vect to_log;
	to_log.logtype = L_SRVSTART;
	to_log.tstamp = (double)t_now.tv_sec+(((double)t_now.tv_usec)/1000000);


	logloop->raw_list.push_back(to_log);

	DEBUG_STREAM << "PLCesall::init_device() before starting plcsrvthread"<<endl;
//starts thread
	srvloop = new plcsrvthread(this);
	srvloop->start();

//	DEBUG_STREAM << "PLCesall::log thread started" << endl;
//debug
//request data from plc for refreshing data at srv startup
/*	usleep(2000000);
	memset(&prepared_cmd, 0, sizeof(intlck_cmd));
	prepared_cmd.actDB = ECHO_MESG_ALM_SYS_CMD;
	write_msg();
	usleep(500000);
	memset(&prepared_cmd, 0, sizeof(intlck_cmd));
	prepared_cmd.actDB = ECHO_MESG_STAT_SYS_CMD;
	write_msg();
	usleep(500000);
	memset(&prepared_cmd, 0, sizeof(intlck_cmd));
	prepared_cmd.actDB = ECHO_MESG_OUT_SYS_CMD;
	write_msg();
	
	usleep(500000);
	memset(&prepared_cmd, 0, sizeof(intlck_cmd));
	prepared_cmd.actDB = ECHO_MESG_ALM_SYS_CMD;
	write_msg();
	usleep(100000);
	memset(&prepared_cmd, 0, sizeof(intlck_cmd));
	prepared_cmd.actDB = ECHO_MESG_OUT_SYS_CMD;
	write_msg();*/
}


//+----------------------------------------------------------------------------
//
// method : 		PLCesall::get_device_property()
// 
// description : 	Read the device properties from database.
//
//-----------------------------------------------------------------------------
void PLCesall::get_device_property()
{
	//	Initialize your default values here (if not done with  POGO).
	//------------------------------------------------------------------

	//	Read device properties from database.(Automatic code generation)
	//------------------------------------------------------------------
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("Connect_param"));
	dev_prop.push_back(Tango::DbDatum("Remote_ip"));
	dev_prop.push_back(Tango::DbDatum("Local_port"));
	dev_prop.push_back(Tango::DbDatum("Db_param"));
	dev_prop.push_back(Tango::DbDatum("Data_config"));
	dev_prop.push_back(Tango::DbDatum("Timeout"));
	dev_prop.push_back(Tango::DbDatum("Id_plc"));

	//	Call database and extract values
	//--------------------------------------------
	if (Tango::Util::instance()->_UseDb==true)
		get_db_device()->get_property(dev_prop);
	Tango::DbDatum	def_prop, cl_prop;
	PLCesallClass	*ds_class =
		(static_cast<PLCesallClass *>(get_device_class()));
	int	i = -1;

	//	Try to initialize Connect_param from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  connect_param;
	else {
		//	Try to initialize Connect_param from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  connect_param;
	}
	//	And try to extract Connect_param value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  connect_param;

	//	Try to initialize Remote_ip from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  remote_ip;
	else {
		//	Try to initialize Remote_ip from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  remote_ip;
	}
	//	And try to extract Remote_ip value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  remote_ip;

	//	Try to initialize Local_port from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  local_port;
	else {
		//	Try to initialize Local_port from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  local_port;
	}
	//	And try to extract Local_port value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  local_port;

	//	Try to initialize Db_param from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  db_param;
	else {
		//	Try to initialize Db_param from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  db_param;
	}
	//	And try to extract Db_param value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  db_param;

	//	Try to initialize Data_config from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  data_config;
	else {
		//	Try to initialize Data_config from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  data_config;
	}
	//	And try to extract Data_config value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  data_config;

	//	Try to initialize Timeout from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  timeout;
	else {
		//	Try to initialize Timeout from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  timeout;
	}
	//	And try to extract Timeout value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  timeout;

	//	Try to initialize Id_plc from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  id_plc;
	else {
		//	Try to initialize Id_plc from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  id_plc;
	}
	//	And try to extract Id_plc value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  id_plc;


	//	End of Automatic code generation
	//------------------------------------------------------------------

}
//+----------------------------------------------------------------------------
//
// method : 		PLCesall::always_executed_hook()
// 
// description : 	method always executed before any command is executed
//
//-----------------------------------------------------------------------------
void PLCesall::always_executed_hook()
{
	
}
//+----------------------------------------------------------------------------
//
// method : 		PLCesall::read_attr_hardware
// 
// description : 	Hardware acquisition for attributes.
//
//-----------------------------------------------------------------------------
void PLCesall::read_attr_hardware(vector<long> &attr_list)
{
	//DEBUG_STREAM << "PLCesall::read_attr_hardware(vector<long> &attr_list) entering... "<< endl;
	//	Add your own code here
}
//+----------------------------------------------------------------------------
//
// method : 		PLCesall::read_Out_system
// 
// description : 	Extract real attribute values for Out_system acquisition result.
//
//-----------------------------------------------------------------------------
void PLCesall::read_Out_system(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCesall::read_Out_system(Tango::Attribute &attr) entering... "<< endl;
	attr_Out_system_read = &msg_outsys[0];
	attr.set_value(attr_Out_system_read, MESG_OUT_SYS_LEN, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCesall::read_Real_system
// 
// description : 	Extract real attribute values for Real_system acquisition result.
//
//-----------------------------------------------------------------------------
void PLCesall::read_Real_system(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCesall::read_Real_system(Tango::Attribute &attr) entering... "<< endl;
	for(int i=0; i<MESG_REAL_SYS_LEN; i++)
		attr_Real_system_read[i] = msg_realsys[i];

	if(plc_status)
		attr.set_value_date_quality(attr_Real_system_read, NULL, Tango::ATTR_VALID, MESG_REAL_SYS_LEN,0,false);
	else
		attr.set_value_date_quality(attr_Real_system_read, NULL, Tango::ATTR_INVALID, MESG_REAL_SYS_LEN,0,false);
}


//+----------------------------------------------------------------------------
//
// method : 		PLCesall::read_Plc_status
// 
// description : 	Extract real attribute values for Plc_status acquisition result.
//
//-----------------------------------------------------------------------------
void PLCesall::read_Plc_status(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCesall::read_Plc_status(Tango::Attribute &attr) entering... "<< endl;
	attr_Plc_status_read = &plc_status;
	attr.set_value(attr_Plc_status_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCesall::read_LogState
// 
// description : 	Extract real attribute values for LogState acquisition result.
//
//-----------------------------------------------------------------------------
void PLCesall::read_LogState(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCesall::read_LogState(Tango::Attribute &attr) entering... "<< endl;
	attr_LogState_read = &logstate;
	attr.set_value(attr_LogState_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCesall::write_LogState
// 
// description : 	Write LogState attribute values to hardware.
//
//-----------------------------------------------------------------------------
void PLCesall::write_LogState(Tango::WAttribute &attr)
{
	//DEBUG_STREAM << "PLCesall::write_LogState(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_LogState_write);
	logstate = attr_LogState_write;
}

//+----------------------------------------------------------------------------
//
// method : 		PLCesall::read_Stat_system
// 
// description : 	Extract real attribute values for Stat_system acquisition result.
//
//-----------------------------------------------------------------------------
void PLCesall::read_Stat_system(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCesall::read_Stat_system(Tango::Attribute &attr) entering... "<< endl;
	attr_Stat_system_read = &msg_statsys[0];
	attr.set_value(attr_Stat_system_read, MESG_STAT_SYS_LEN, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCesall::read_Alarm_system
// 
// description : 	Extract real attribute values for Alarm_system acquisition result.
//
//-----------------------------------------------------------------------------
void PLCesall::read_Alarm_system(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCesall::read_Alarm_system(Tango::Attribute &attr) entering... "<< endl;
	attr_Alarm_system_read = &msg_almsys[0];
	attr.set_value(attr_Alarm_system_read, MESG_ALM_SYS_LEN, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		PLCesall::read_Db_alarm
// 
// description : 	Extract real attribute values for Db_alarm acquisition result.
//
//-----------------------------------------------------------------------------
void PLCesall::read_Db_alarm(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "PLCesall::read_Db_alarm(Tango::Attribute &attr) entering... "<< endl;
	attr_Db_alarm_read = &alarm_db[0];

	timeval taim;
	double fractpart, intpart;

  	fractpart = modf (timestamp_alm , &intpart);
	taim.tv_sec = (__time_t)intpart;
	taim.tv_usec = (__suseconds_t)fractpart*1000000;
	
	if(timestamp_alm!=0){
		attr.set_value_date_quality(attr_Db_alarm_read, taim, Tango::ATTR_VALID, MESG_ALM_SYS_LEN,0,false);
	}
	else{
		attr.set_value_date_quality(attr_Db_alarm_read, taim, Tango::ATTR_INVALID, MESG_ALM_SYS_LEN,0,false);
	}
}

//+----------------------------------------------------------------------------
//
// method : 		PLCesall::write_Db_alarm
// 
// description : 	Write Db_alarm attribute values to hardware.
//
//-----------------------------------------------------------------------------
void PLCesall::write_Db_alarm(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "PLCesall::write_Db_alarm(Tango::WAttribute &attr) entering... "<< endl;
	for(int i=0;i<MESG_ALM_SYS_LEN;i++){//no ack
		w_alarm_db[i] = 0;
	}
	attr.get_write_value((const Tango::DevDouble*&)attr_Db_alarm_write);
	for(int i=0;i<MESG_ALM_SYS_LEN;i++,attr_Db_alarm_write++){
		w_alarm_db[i] = *attr_Db_alarm_write;
	}
//	DEBUG_STREAM << "written value: " << w_alarm_53[0]<<"-"<< w_alarm_53[1]<<endl;
/*	memset(&prepared_cmd, 0, sizeof(intlck_cmd));
	prepared_cmd.actDB = ACK_CMD;
	int res;
	for(int i=0;i<MESG_53_LEN_W;i++){//oppure MESG_53_DATA?
		for(int j=0;j<8;j++){
			if(w_alarm_53[8*i+j]!=0) {
				bcopy(&(prepared_cmd.dw[i]), &res, 1);
				res |= maskbits[j];
				res &= 0x00ff;
				bcopy(&res, &(prepared_cmd.dw[i]), 1);
			}
		}
	}

	write_msg();
	timeval t_now;
	gettimeofday(&t_now,NULL);//takes timestamp
	timest_vect to_log;
	to_log.logtype = L_ACK;
	to_log.tstamp = (double)t_now.tv_sec+(((double)t_now.tv_usec)/1000000);

	mutex->lock();
	raw_list.push_back(to_log);
	mutex->unlock();
*/
	DEBUG_STREAM << "PLCesall::write_Db_alarm(): done "<< endl;
}

//+------------------------------------------------------------------
/**
 *	method:	PLCesall::init_database
 *
 *	description:	
 *
 *
 */
//+------------------------------------------------------------------
void PLCesall::init_database()
{
//cout<<"ok"<<endl;
	if(!mysql_init(&dbp)){
		set_state(Tango::FAULT);
		set_status("init db log error");
		INFO_STREAM << "PLCesall::init_device() init db log error" << endl;
	}
//cout<<"ok2"<<endl;
	my_bool my_auto_reconnect=1;
	if(mysql_options(&dbp,MYSQL_OPT_RECONNECT,&my_auto_reconnect) !=0){
		set_state(Tango::FAULT);
		set_status("mysql auto reconnection error");
		INFO_STREAM << "PLCesall::init_device() mysql auto reconnection error" << mysql_error(&dbp) << endl;
	}
//cout<<"ok3"<<endl;
	string dbhost;
	string dbuser;
	string dbpw;
	string dbname;
	string dbport;
	int dbportint;
	vector<string>::iterator iter = db_param.begin();
	dbhost = ((string)*iter);
	iter++;
	dbuser = ((string)*iter);
	iter++;
	dbpw = ((string)*iter);
	iter++;
	dbname = ((string)*iter);
	iter++;
	dbport = ((string)*iter);
	dbportint = atoi(dbport.c_str());
//cout<<"ok4"<<endl;
	if(dbportint!=0){
		if(!mysql_real_connect(&dbp, dbhost.c_str(), dbuser.c_str(), dbpw.c_str(), dbname.c_str(), dbportint, NULL, 0)){
//		if(!mysql_real_connect(&dbp, "srv-log-srf.fcs.elettra.trieste.it", "root", "milmi24", "intlkfermi", 3306, NULL, 0)){
//cout<<"ok5"<<endl;
			set_state(Tango::FAULT);
			set_status("db log connection error");
			INFO_STREAM << "PLCesall::init_device() db log connection error" << endl;
		}
		else{
			set_state(Tango::ON);
			set_status("db log connected");
			LOG_DB_NAME = dbname;
		}
	}
	else{
//cout<<"ok6"<<endl;
		set_state(Tango::FAULT);
		set_status("db log connection error");
		INFO_STREAM << "PLCesall::init_device() db log connection error" << endl;
	}
}



//+------------------------------------------------------------------
/**
 *	method:	PLCesall::send_cmd
 *
 *	description:	method to execute "Send_cmd"
 *
 * @param	argin	Command code, cmd argument
 *
 */
//+------------------------------------------------------------------
void PLCesall::send_cmd(const Tango::DevVarShortArray *argin)
{
	DEBUG_STREAM << "PLCesall::send_cmd(): entering... !" << endl;

	//	Add your own code to control device here
	DEBUG_STREAM<<"cmd "<<argin->length()<< " code: "<<(*argin)[0]<<endl;
	memset(&prepared_cmd, 0, sizeof(intlck_cmd));
//	prepared_cmd.actDB = 0;
	int code = (*argin)[0];
	int parm;
	int16_t val;
	switch(code){
		case ECHO_MESG_STAT_SYS_CMD:
			DEBUG_STREAM<<"PLCesall::send_cmd(): sending cmd: "<<ECHO_MESG_STAT_SYS_CMD<<endl;
			prepared_cmd.actDB = ECHO_MESG_STAT_SYS_CMD;
		break;
		case ECHO_MESG_ALM_SYS_CMD:
			DEBUG_STREAM<<"PLCesall::send_cmd(): sending cmd: "<<ECHO_MESG_ALM_SYS_CMD<<endl;
			prepared_cmd.actDB = ECHO_MESG_ALM_SYS_CMD;
		break;
		case ECHO_MESG_OUT_SYS_CMD:
			DEBUG_STREAM<<"PLCesall::send_cmd(): sending cmd: "<<ECHO_MESG_OUT_SYS_CMD<<endl;
			prepared_cmd.actDB = ECHO_MESG_OUT_SYS_CMD;
		break;
		case SYNC_CODE:
			DEBUG_STREAM<<"PLCesall::send_cmd(): sending cmd: "<<SYNC_CODE<<endl;
			prepared_cmd.actDB = SYNC_CODE;
			syncronize();
			return;
		break;
		case OPN_CLS_CMD_FEL1:
		case OPN_CLS_CMD_FEL2:
		case OPN_CLS_CMD_POS:
		case OPN_CLS_CMD_MAG:
		case SET_PERCENT_INTLK_THRESH:
		case PMP_VEL_CMD_FEL1:
		case PMP_VEL_CMD_FEL2:
		case PPM_START_STOP_CMD_FEL1:
		case PPM_START_STOP_CMD_FEL2:
		case OPN_CLS_CMD_FEL:
			DEBUG_STREAM<<"PLCesall::send_cmd(): sending cmd opn-cls: "<<code<<" arg: "<<(*argin)[1] <<endl;
			memset(&prepared_cmd, 0, sizeof(intlck_cmd));
			prepared_cmd.actDB = code;
			parm = (*argin)[1];
			if((parm<CMD_OPN_VLV_PFE_F0102_01) || (parm>MAX_COMMAND))
			{
				DEBUG_STREAM <<"PLCvacll::send_cmd(): wrong parameter "<<parm<<endl;
			}
			else{
				DEBUG_STREAM<<"PLCvacll::send_cmd(): ----sending block "<<blkcodes[parm]<<" mask 0x"<<hex<<maskcodes[parm]<<dec<<endl;
				if(code == SET_PERCENT_INTLK_THRESH && parm==CMD_SET_PERCENT_INTLK_THRESH)	//808 and 72
				{
					val = (*argin)[2];
					if((val<-100) || (val>100))
					{
						DEBUG_STREAM <<"PLCvacll::send_cmd(): wrong parameter setting interlock threshold cmd="<<code<<" parm="<<parm<<" val="<<val<<endl;
					}
					else{
						*((uint16_t *)&prepared_cmd.cmd[blkcodes[parm]])= htons(val);
						write_msg();
					}
					return;
				}
				prepared_cmd.cmd[blkcodes[parm]]= maskcodes[parm];
				timeval t_now;
				gettimeofday(&t_now,NULL);//takes timestamp
				write_msg();
				timest_vect to_log;
				to_log.logtype = L_CMD;
				to_log.id_cmd = parm;
				to_log.id_fel = code;
				if(code == PMP_VEL_CMD_FEL1 || code == PPM_START_STOP_CMD_FEL1)
					to_log.id_fel = OPN_CLS_CMD_FEL1;
				if(code == PMP_VEL_CMD_FEL2 || code == PPM_START_STOP_CMD_FEL2)
					to_log.id_fel = OPN_CLS_CMD_FEL2;
				if(code == OPN_CLS_CMD_FEL)
					to_log.id_fel = SET_PERCENT_INTLK_THRESH;
				to_log.tstamp = (double)t_now.tv_sec+(((double)t_now.tv_usec)/1000000);
				logloop->raw_list.push_back(to_log);
			}
			return;
		break;
#if 0
		case SET_PERCENT_INTLK_THRESH:
			DEBUG_STREAM<<"PLCesall::send_cmd(): sending cmd set % intlk thresh: "<<code<<" arg: "<<(*argin)[1] <<endl;
			memset(&prepared_cmd, 0, sizeof(intlck_cmd));
			prepared_cmd.actDB = code;
			val = (*argin)[1];
			if((val<-100) || (val>100))
			{
				DEBUG_STREAM <<"PLCvacll::send_cmd(): wrong parameter cmd="<<code<<" val="<<val<<endl;
			}
			else{
				*((uint16_t *)&prepared_cmd.cmd[16])= htons(val);
				write_msg();
				/*timeval t_now;	//TODO: log
				gettimeofday(&t_now,NULL);//takes timestamp
				timest_vect to_log;
				to_log.logtype = L_CMD;
				to_log.id_cmd = parm;
				to_log.id_fel = code;
				if(code == PMP_VEL_CMD_FEL1 || code == PPM_START_STOP_CMD_FEL1)
					to_log.id_fel = OPN_CLS_CMD_FEL1;
				if(code == PMP_VEL_CMD_FEL2 || code == PPM_START_STOP_CMD_FEL2)
					to_log.id_fel = OPN_CLS_CMD_FEL2;
				to_log.tstamp = (double)t_now.tv_sec+(((double)t_now.tv_usec)/1000000);
				logloop->raw_list.push_back(to_log);*/
			}
			return;
			break;
#endif
		case ACK_CMD:
			DEBUG_STREAM<<"PLCesall::send_cmd(): sending cmd ack: "<<ACK_CMD<<endl;
			memset(prepared_cmd.ack, 0xff, 244);
			prepared_cmd.actDB = ACK_CMD;
			timeval t_now;
			gettimeofday(&t_now,NULL);//takes timestamp
			timest_vect to_log;
			to_log.logtype = L_ACK;
			to_log.tstamp = (double)t_now.tv_sec+(((double)t_now.tv_usec)/1000000);
			logloop->raw_list.push_back(to_log);
		break;
		default:
			INFO_STREAM<<"PLCesall::send_cmd(): cmd not supported: "<<code<<endl;
			return;
		break;
	}
	write_msg();

}

//+------------------------------------------------------------------
/**
 *	method:	PLCesall::acknowledge
 *
 *	description:	method to execute "Acknowledge"
 *
 * @param	argin	
 *
 */
//+------------------------------------------------------------------
void PLCesall::acknowledge(const Tango::DevVarShortArray *argin)
{
	DEBUG_STREAM << "PLCesall::acknowledge(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&prepared_cmd, 0x0, sizeof(intlck_cmd));
	memset(prepared_cmd.ack, 0xff, 244);//TODO: prepared_cmd.ack[...] = 1 << ...;
	//TODO: prepared_cmd.ack[...] = 1 << ...;
	prepared_cmd.actDB = ACK_CMD;

	// lz added try catch
	try {
		write_msg();
		timeval t_now;
		gettimeofday(&t_now,NULL);//takes timestamp
		timest_vect to_log;
		to_log.logtype = L_ACK;
		to_log.tstamp = (double)t_now.tv_sec+(((double)t_now.tv_usec)/1000000);

		logloop->raw_list.push_back(to_log);
	}
	catch (Tango::DevFailed e) {
		Tango::Except::re_throw_exception(e,
			(const char *) "write error",
			(const char *) "write cmd error",
			(const char *) "PLCintll::acknowledge()", Tango::ERR);					
	}

}



//+------------------------------------------------------------------
/**
 *	method:	PLCesall::update_db_description
 *
 *	description:	method to execute "UpdateDBDescription"
 *
 * @param	argin	
 *
 */
//+------------------------------------------------------------------
void PLCesall::update_db_description()
{
	DEBUG_STREAM << "PLCesall::update_db_description(): entering... !" << endl;
	timeval t_now;
	//	Add your own code to control device here
	try {
		load_cond();
		gettimeofday(&t_now,NULL);//takes timestamp
		timest_vect to_log;
		to_log.logtype = L_UPDATE;
		to_log.tstamp = (double)t_now.tv_sec+(((double)t_now.tv_usec)/1000000);

		logloop->raw_list.push_back(to_log);
		//init_attributeprop();	//moved in thread
	}
	catch (Tango::DevFailed e)
	{
		INFO_STREAM << "PLCesall::update_db_description(): error updating description="<<e.errors[0].desc << endl;
		Tango::Except::re_throw_exception(e,
			(const char *) "update db error",
			(const char *) "update db error",
			(const char *) "PLCesall::update_db_description()", Tango::ERR);
	}

}

//+------------------------------------------------------------------
/**
 *	method:	PLCesall::write_msg
 *
 *	description:	method to execute "write_msg"
 *
 *
 */
//+------------------------------------------------------------------
void PLCesall::write_msg()
{
	DEBUG_STREAM << "PLCesall::write_msg(): entering... !" << endl;

	//	Add your own code to control device here
	unsigned char query[BUF_LEN];
        memset (query,0,BUF_LEN);

	try{
	//write the message to PLC 
	
		prepared_cmd.actDB = htons(prepared_cmd.actDB);
		memcpy(query, &prepared_cmd, sizeof(intlck_cmd));	
		DEBUG_STREAM <<"PLCesall::write_msg: size: "<< sizeof(intlck_cmd)<< endl;

		for(int k=0;k<sizeof(intlck_cmd);)
		{
			stringstream tmp;
			int g;
			tmp << "dbTX: "<<setw(3)<<setfill(' ')<<k<<" -> ";
			for(g=0; g<10; g++)
			{
				if(k+g>=sizeof(intlck_cmd))
					break;
				tmp << " " <<hex<<setw(2)<<setfill(' ')<< (unsigned int)query[k+g];
			}
			k+=g;
			if(k<sizeof(intlck_cmd))
			{
				tmp << " |  "<<dec<<setw(3)<<setfill(' ')<<k<<hex<<" -> ";
				for(g=0; g<10; g++)
				{
					if(k+g>=sizeof(intlck_cmd))
						break;
					tmp << " " <<hex<<setw(2)<<setfill(' ')<< (unsigned int)query[k+g];
				}
				k+=g;
			}
			DEBUG_STREAM<<tmp.str()<<dec<<endl;
		}

		send_frame(query, (sizeof(intlck_cmd)));

	}
	catch(Tango::DevFailed e) {
		set_state(Tango::FAULT);
		set_status("Write command failed");
		Tango::Except::re_throw_exception(e,
			(const char *) "write error",
			(const char *) "write cmd error",
			(const char *) "PLCesall::send_cmd", Tango::ERR);					
	}
	

	return;

}

//+------------------------------------------------------------------
/**
 *	method:	PLCesall::send_frame()
 *
 * Description:	Send a  frame to a node via  TCP/IP
 *
 * Arg(s) In:	unsigned char *query -  data to send
 *		short query_length - number of data words to send
 *
 *
 */
//+------------------------------------------------------------------

void PLCesall::send_frame (unsigned char *query, short query_length)
{
	unsigned char frame[BUF_LEN];
   	memset ( frame, 0, sizeof(frame) );
/*	unsigned short int sbuf[BUF_LEN];
        memset (sbuf,0,1+(2*query_length));
	memcpy(&sbuf, &query[0], 2*query_length);
//	for (int i=0; i<query_length; i++){
//		sbuf[i] = htons(sbuf[i]);
//	}
	memcpy(&frame[0], &sbuf, 2*query_length);*/
	memcpy(&frame[0], &query[0], query_length);
		
DEBUG_STREAM<<"query length: "<< query_length<<endl;
/*TMP!!!!!!!!!!!*///return;			
   	int writecnt = write (new_sd, frame,query_length);
	if(writecnt ==-1){
		stringstream tmp;
		tmp << "error writing socket: "<<strerror(errno) << ends;
		set_state(Tango::UNKNOWN);
		set_status(tmp.str());
		INFO_STREAM << "PLCesall::send_frame(): " << tmp.str() << endl;
				Tango::Except::throw_exception(
					(const char *) "write error",
					tmp.str(),
					(const char *) "PLCesall::send_frame", Tango::ERR);			
	}
	else
		DEBUG_STREAM << "PLCesall::send_frame(): written " << writecnt << " chars" <<endl;	
}

uint16_t PLCesall::plc_swap(uint16_t in)
{
	return ((in>>8)|((in&0x00ff)<<8));
}


void PLCesall::syncronize()
{
	DEBUG_STREAM << "PLCesall::syncronize(): entering... !" << endl;

	//	Add your own code to control device here
	memset(&prepared_cmd, 0x0000, sizeof(intlck_cmd));
	prepared_cmd.actDB = SYNC_CODE;
	timeval seco;
	time_t tim=time(NULL);
	gettimeofday(&seco,NULL);
	tm *now=localtime(&tim);
	char buf[10];
	char buf2[10];
	
	u_short mon = now->tm_mon+1;
	u_short year = now->tm_year-100;
	u_short hour = now->tm_hour;
	u_short day = now->tm_mday;
	u_short sec = now->tm_sec;
	u_short min = now->tm_min;
	u_short wday = now->tm_wday+1;

  	u_short msec = (u_short)(((double)seco.tv_usec)/10000);
  	u_short msec1 = (u_short)(((double)seco.tv_usec)/1000);
	bcopy(&mon, &(prepared_cmd.month), 1);
	bcopy(&year, &(prepared_cmd.year), 1);
	bcopy(&hour, &(prepared_cmd.hour), 1);
	bcopy(&day, &(prepared_cmd.day), 1);
	bcopy(&sec, &(prepared_cmd.sec), 1);
	bcopy(&min, &(prepared_cmd.min), 1);
	bcopy(&wday, &(prepared_cmd.wday), 1);
	bcopy(&msec, &(prepared_cmd.msec), 1);

	sprintf(buf, "%02d", (now->tm_year-100));
	prepared_cmd.year = (buf[0]-'0'<<4)+buf[1]-'0';
        memset (buf,0,sizeof(buf));
	sprintf(buf, "%02d", (now->tm_mon+1));
	prepared_cmd.month = (buf[0]-'0'<<4)+buf[1]-'0';
        memset (buf,0,sizeof(buf));
	sprintf(buf, "%02d", (now->tm_mday));
	prepared_cmd.day = (buf[0]-'0'<<4)+buf[1]-'0';
        memset (buf,0,sizeof(buf));
	sprintf(buf, "%02d", (now->tm_hour));
	prepared_cmd.hour = (buf[0]-'0'<<4)+buf[1]-'0';
        memset (buf,0,sizeof(buf));
	sprintf(buf, "%02d", (now->tm_min));
	prepared_cmd.min = (buf[0]-'0'<<4)+buf[1]-'0';
        memset (buf,0,sizeof(buf));
	sprintf(buf, "%02d", (now->tm_sec));
	prepared_cmd.sec = (buf[0]-'0'<<4)+buf[1]-'0';
        memset (buf,0,sizeof(buf));
	sprintf(buf, "%01d", (now->tm_wday+1));
        memset (buf2,0,sizeof(buf2));
	sprintf(buf2, "%03d", (msec1));
	prepared_cmd.wday = ((buf2[2]-'0'<<4)+buf[0]-'0')&0x00ffff;
        memset (buf,0,sizeof(buf));
	sprintf(buf, "%02d", (msec));
	prepared_cmd.msec = (buf[0]-'0'<<4)+buf[1]-'0';

	DEBUG_STREAM <<"setting PLC time to:"<< endl;
	DEBUG_STREAM <<"year:"<<year <<" month:"<<mon <<" day:"<<day <<" hour:"<<hour <<" min:"<< min<<" sec:"<<sec<<" msec:"<<msec<<" msec1:"<<msec1<<" usec="<< seco.tv_usec<<" buf2="<<buf2<< endl;
	DEBUG_STREAM <<"mesg len:"<<sizeof(intlck_cmd) << endl;
/*	unsigned short int dbuf[4];
	memset (dbuf,0,8);
	bcopy(&(prepared_cmd.month), &dbuf[0], 8);
	for (int i=0; i<4; i++){
		dbuf[i] = plc_swap(dbuf[i]);
	}
	bcopy(&(dbuf[0]), &(prepared_cmd.month), 8);*/

	write_msg();

}

void PLCesall::load_cond(){
	fd.open(start_file);  

	char pbuf[128];		
	if (!fd){
		DEBUG_STREAM << "PLCesall::load_cond() cannot open file:" << strerror(errno) << endl;
   	}
	else{
		DEBUG_STREAM << "PLCesall::load_cond() file opened successfully" << endl;
		string lineread, typ, pos, nam, siz, def, val;//typ=attr nam=label val=descr
		int k=0;
//		int linestatok = 0;
//		int linealmok = 0;
//		int lineattrok = 0;
 //		int linecmdok = 0;
 //		int linedbstringok = 0;
 //		int linedbboolok = 0;
// 		int linenameok = 0;
	       	int n;
		while(std::getline(fd, lineread, '\n')){ // Read line by line
			n = 0;
			k++;
			memset(pbuf, 0, 128);
			typ = lineread.substr(0, lineread.find('\t'));
			lineread = lineread.substr(lineread.find('\t')+1, lineread.length());
			pos = lineread.substr(0, lineread.find('\t'));
			lineread = lineread.substr(lineread.find('\t')+1, lineread.length());
			nam = lineread.substr(0, lineread.find('\t'));
			lineread = lineread.substr(lineread.find('\t')+1, lineread.length());
			siz = lineread.substr(0, lineread.find('\t'));
			lineread = lineread.substr(lineread.find('\t')+1, lineread.length());
			size_t pos_def = lineread.find('\t');
			if(pos_def != string::npos)
			{
				def = lineread.substr(0, lineread.find('\t'));
				lineread = lineread.substr(lineread.find('\t')+1, lineread.length());
				val = lineread.substr(lineread.find('\t')+1, lineread.length());
			}
			else
			{
				def = lineread;
				val = "";
			}
			if((int)(typ.find("db50"))!=-1){//h=0
//DEBUG_STREAM<<"PLCesall::load_cond() 50-typ "<<typ.c_str()<<"- pos "<<pos.c_str()<<"- nam "<<nam.c_str()<<"- siz "<<siz.c_str()<<"- def "<<def.c_str()<<endl;
						int p1,p2;
						p1 = p2 = 0;
						sscanf(pos.c_str(), "%d.%d", &p1, &p2);
						if(p1 >= 10)
						{
							(val_labels[0])[8*(p1-10)+p2] = nam;//h=0
							(val_pos[0])[8*(p1-10)+p2] = val;//h=0
						}
			}else if((int)(typ.find("db53"))!=-1){//h=1
//DEBUG_STREAM<<"PLCesall::load_cond() 53-typ "<<typ.c_str()<<"- pos "<<pos.c_str()<<"- nam "<<nam.c_str()<<"- siz "<<siz.c_str()<<"- def "<<def.c_str()<<endl;
						int p1,p2;
						p1 = p2 = 0;
						sscanf(pos.c_str(), "%d.%d", &p1, &p2);
						if((typ.find("N")==string::npos) && (p1 >= 10))
						{
							(val_labels[1])[8*(p1-10)+p2] = nam;//h=1
							(val_pos[1])[8*(p1-10)+p2] = val;//h=1
							(val_labels[2])[8*(p1-10)+p2] = nam;//h=2
							(val_pos[2])[8*(p1-10)+p2] = val;//h=2
							alm_active[(8*(p1-10)+p2)] = true;
						}
//					if((typ.find("N")==string::npos) && (p1 >= 10))alm_active[(8*(p1-10)+p2)] = true;
			}else if((int)(typ.find("db55"))!=-1){//h=1
//DEBUG_STREAM<<"PLCesall::load_cond() 55-typ "<<typ.c_str()<<"- pos "<<pos.c_str()<<"- nam "<<nam.c_str()<<"- siz "<<siz.c_str()<<"- def "<<def.c_str()<<endl;
						int p1,p2;
						p1 = p2 = 0;
						sscanf(pos.c_str(), "%d.%d", &p1, &p2);
						if(p1 >= 10)
						{
							(val_labels[4])[8*(p1-10)+p2] = nam;
							(val_pos[4])[8*(p1-10)+p2] = val;
						}
//					if((typ.find("N")==string::npos) && (p1 >= 10))alm_active[(8*(p1-10)+p2)] = true;
			}
		}
		DEBUG_STREAM << "PLCesall::load_cond() finished successfully" << endl;
   	}
	fd.close();
	
	
}

//+------------------------------------------------------------------
/**
 *	method:	PLCesall::init_attributeprop
 *
 *	description:	
 *
 *
 */
//+------------------------------------------------------------------
void PLCesall::init_attributeprop()
{
	DEBUG_STREAM << "PLCesall::init_attributeprop() entering" << endl;
	Tango::Database *db = new Tango::Database();
	vector <Tango::DbData> vdb_datastat;
	for(int i=0;i<5;i++){
		vector <string> lastat;
		vector <string> destat;
		for (int k = 0; k < att_len[i]; k++){
			lastat.push_back(string((val_labels[i])[k]));
			destat.push_back(string((val_pos[i])[k]));
		}	
		Tango::DbData db_datastat;
		Tango::DbDatum db_stat(att_names[i]), lab_stat("labels"), des_stat("positions");
		db_stat << (short int) 2;
		db_datastat.push_back(db_stat);
		lab_stat << lastat;
		des_stat << destat;
		db_datastat.push_back(lab_stat);
		db_datastat.push_back(des_stat);
		vdb_datastat.push_back(db_datastat);
	}
	vector<Tango::DbData>::iterator ite;

	try {
		db->set_timeout_millis(10000);	
		for( ite=vdb_datastat.begin() ; ite < vdb_datastat.end(); ite++ ){
			db->put_device_attribute_property(get_name(), *ite);
			//get_db_device()->put_attribute_property(*ite);
		}
	} catch (Tango::DevFailed &e) {
		INFO_STREAM<<"init PLCesall::init_attributeprop: exception="<<e.errors[0].desc<<endl;
		Tango::Except::print_exception(e);
	}
	DEBUG_STREAM << "PLCesall::init_attributeprop() exiting" << endl;
}

long int PLCesall::calc_timeval(u_char year,u_char month,u_char day,u_char hour,u_char min,u_char sec,u_char msec,u_char wday)
{
	char buffer [10];
	tm t2;
    time_t tn;

	
    memset (buffer,0,10);
	sprintf (buffer, "%x", sec);
	t2.tm_sec = atoi(buffer);
	DEBUG_STREAM << "PLCesall::calc_timeval: t2.tm_sec="<<(int)t2.tm_sec;
        memset (buffer,0,10);
	sprintf (buffer, "%x", min);
	t2.tm_min = atoi(buffer);
	DEBUG_STREAM << "PLCesall::calc_timeval: t2.tm_min="<<(int)t2.tm_min;
        memset (buffer,0,10);
	sprintf (buffer, "%x", hour);
	t2.tm_hour = atoi(buffer);
	DEBUG_STREAM << "PLCesall::calc_timeval: t2.tm_hour="<<(int)t2.tm_hour;
        memset (buffer,0,10);
	sprintf (buffer, "%x", day);
	t2.tm_mday = atoi(buffer);
	DEBUG_STREAM << "PLCesall::calc_timeval: t2.tm_mday="<<(int)t2.tm_mday;
        memset (buffer,0,10);
	sprintf (buffer, "%x", month);
	t2.tm_mon = atoi(buffer)-1;
	DEBUG_STREAM << "PLCesall::calc_timeval: t2.tm_mon="<<(int)t2.tm_mon;
        memset (buffer,0,10);
	sprintf (buffer, "%x", year);
	t2.tm_year = atoi(buffer)+100;
	DEBUG_STREAM << "PLCesall::calc_timeval: t2.tm_year="<<(int)t2.tm_year;
        memset (buffer,0,10);
	sprintf (buffer, "%x", (wday&0xf));
	t2.tm_wday = atoi(buffer)-1;
	DEBUG_STREAM << "PLCesall::calc_timeval: t2.tm_wday="<<(int)t2.tm_wday;
	//0: consider time to convert never dst, time converted automatically adjusted with dst
	//-1: consider time to convert already ok with dst, so no adjustment applied by mktime
	t2.tm_isdst = 0/*-1*/;

	DEBUG_STREAM << __func__<<": year=0x"<<hex<<(int)year<<" month=0x"<<(int)month<<" day=0x"<<(int)day<<" hour=0x"<<(int)hour<<" min=0x"<<(int)min<<" sec=0x"<<(int)sec<<dec<<endl;
	tn = mktime(&t2)/*-timezone*/;//with -timezone as if GMT
	DEBUG_STREAM << __func__<<": --> tn="<<tn<<" timezone="<<timezone<<" isdst="<<t2.tm_isdst<<endl;
	return  (long int)tn;
}	

long int PLCesall::calc_time_ms(u_char wday,u_char msec)
{
	char buffer [10];
	long d = 0;
		long ms_new;
		int c1, c2;
		ms_new = 0;
        	memset (buffer,0,10);
		sprintf (buffer, "%x", msec);
		c1 = atoi(buffer);
        	memset (buffer,0,10);
		sprintf (buffer, "%x", (wday>>4));
		c2 = atoi(buffer);
		ms_new = c1*10+c2;
		d = ms_new;
	
	return d;

}



}	//	namespace
